/*
    Copyright 2015 Carlos Toro Peas, Madrid, Espaa 
    This file is part of Android Integral Calculator. 
    Inherted from MaximaOnAndroid project whose
    Copyright 2012, Yasuaki Honda (yasuaki.honda@gmail.com)
    
    Android Integral Calculator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Android Integral Calculator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.carlos.math.functions;

import com.numerico.algebralineal.Util;

public class AnalyticKernel extends VariabledFunction
{
	private double R;
	public static final String KERNEL_POISSON = "(1-r^2)/((2*" + Util.PI + ")*(1-2*r*(cos(z)) + r^2))";
	public static final String KERNEL_GAUSS = "(1-r^2)/(2*" + Util.PI + ")*(1-2*r*(cos(z)) + r^2)";
	public static final String KERNEL_DIRICHLET = "(1-r^2)/(2*" + Util.PI + ")*(1-2*r*(cos(z)) + r^2)";
	public static final String KERNEL_FLEJER = "(1-r^2)/(2*" + Util.PI + ")*(1-2*r*(cos(z)) + r^2)";
	public static final String KERNEL_GREEN = "(1-r^2)*(1-2*r*(cos(z)) + r^2)/(2*" + Util.PI + ")";
	private VariabledFunction evaluate;
	public AnalyticKernel() {
		
	}
	public static VariabledFunction getKernelFunction(String kernel)
	{
		VariabledFunction n = new VariabledFunction(kernel);
		n.addVariable("r",0);
		return n;
	}
	
	public static VariabledFunction getConvolutionKernel(VariabledFunction f, String kernel)
	{
		return getKernelFunction(kernel).convolues("z", "x", f);
	}
	
	
	public static double appliesKernelAndEvaluatesAt(VariabledFunction f, double[] values, Interval interval, String kernel) throws Exception
	{
		VariabledFunction app = getKernelFunction(kernel).convolues("z", "x", f);
		app = app.evaluateAtVariable("r", values[0]);
		app = app.evaluateAtVariable("z", values[1]);
		//esta correcto, la integral es dx
		return app.integrates(interval);
	}
	
	public static double evaluatesAt(VariabledFunction convoluedFunction, double[] values, Interval interval) throws Exception
	{
		convoluedFunction = convoluedFunction.evaluateAtVariable("r", values[0]);
		convoluedFunction = convoluedFunction.evaluateAtVariable("z", values[1]);
		//esta correcto, la integral es dx
		return convoluedFunction.integrates(interval);
	}
	
	
	public void setFunction(String f)
	{
		evaluate = new VariabledFunction("(" + f + ")*(" + KERNEL_POISSON + ")");
	}
	public double evaluate(double r, double z) throws Exception
	{
		return evaluate.f(new double[]{r, z})[0];
	}
	
	public double getR() {
		return R;
	}
	public void setR(double r) {
		R = r;
	}
	
}
	
