
/*
    Copyright 2015 Carlos Toro Peas, Madrid, Espaa 
    This file is part of Android Integral Calculator. 
    Inherted from MaximaOnAndroid project whose
    Copyright 2012, Yasuaki Honda (yasuaki.honda@gmail.com)
    
    Android Integral Calculator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Android Integral Calculator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.carlos.math.functions;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

public class ExtendedFucntion extends VariabledFunction
{

	private List<Double> nodes;
	private List<VariabledFunction> functions;
	
	public ExtendedFucntion(List<VariabledFunction> sortedFunctions, List<Double> sortedNodes) 
	{
		super();
		this.dimension = 1;
		this.functions = new ArrayList<VariabledFunction>();
		for (VariabledFunction f: sortedFunctions)
		{
			this.addFunction(f.getFunctionAt(0));
		}
		
		this.functions = sortedFunctions;
		nodes = sortedNodes;
	}

		
	public void clear()
	{
		this.nodes.clear();
		this.functions.clear();
	}
	public ExtendedFucntion(Vector<String> sortedFunctions, Vector<Double> sortedNodes) 
	{
		super();
		this.dimension = 1;
		this.functions = new ArrayList<VariabledFunction>();
		nodes = new ArrayList<Double>();
		for(String f : sortedFunctions)
		{
			this.addFunction(f);
		}
		for(Double n : sortedNodes)
		{
			this.nodes.add(n);
		}
	}
	
	public ExtendedFucntion(String []sortedFunctions, double []sortedNodes) 
	{
		super();
		this.dimension = 1;
		this.functions = new ArrayList<VariabledFunction>();
		nodes = new ArrayList<Double>();
		for(String f : sortedFunctions)
		{
			this.addFunction(f);
		}
		for(Double n : sortedNodes)
		{
			this.nodes.add(n);
		}
	}
	
	public static void main (String []args) throws Exception
	{
		double[] d = new double[]{-1, 0, 1};
		
		ExtendedFucntion f = new ExtendedFucntion(new String[]{"x", "x^2"}, d);
	}
	
	public ExtendedFucntion differenciate(String variable)
	{
		ExtendedFucntion ef ;
		List<VariabledFunction> derivates = new ArrayList<VariabledFunction>();
		
		for(VariabledFunction f : this.functions)
		{
			derivates.add(f.differenciate(variable));
		}
		
		return new ExtendedFucntion(derivates, this.nodes);
	}
	
	public double integrate() throws Exception
	{
		double limit_inf, limit_sup, result = 0;
		limit_inf = this.nodes.get(0);
		limit_sup = this.nodes.get(1);
		int k = 0;
		for(VariabledFunction f: this.functions)
		{
			limit_sup = this.nodes.get(k+1);
			result += f.integrates(limit_inf, limit_sup);
			limit_inf = limit_sup;
			k++;
		}
		
		return result;
	}
	
	public void addFunction (String function)
	{
		this.functions.add(new VariabledFunction(function));
	}
	
	
	public void removeFunction(int index)
	{
		if(functions.size() > index)
		{
			this.functions.remove(index);
			this.nodes.remove(index-1);
		}
	}
	public double [] f(double x, double[] y)
	{
		Interval i = null;
		VariabledFunction f = null;
		
		for(int k=1; k< nodes.size(); k++)
		{
			f = functions.get(k-1);
			
			i = new Interval(nodes.get(k-1), nodes.get(k));
			if(i.belongsToMe(x))
			{
				return f.f(x, y);
			}
			
		}
		
		return y;
	}
	
	public Vector<Vector<String>> getTableVector()
	{
		Vector<Vector<String>> v = new Vector<Vector<String>>();
		Vector<String> w = null;
		int k = 0;
		for(VariabledFunction f : this.functions)
		{
			w = new Vector<String>();
			w.add(f.toString());
			w.add(Double.toString(this.nodes.get(k)));
			w.add(Double.toString(this.nodes.get(k+1)));
			v.add(w);
			k++;
		}
		return v;
	}

	@Override	
	public boolean isExtendedFunction()
	{
		return true;
	}
	
	public int getFunctionIndexForX(double x)
	{
		Interval i = null;
		VariabledFunction f = null;
		
		for(int k=1; k< nodes.size(); k++)
		{
			f = functions.get(k-1);
			i = new Interval(nodes.get(k-1), nodes.get(k));
			if(i.belongsToMe(x))
			{
				return k-1;
			}
		}
		return 0;
	}
	public VariabledFunction getFunctionForX(double x)
	{
		Interval i = null;
		VariabledFunction f = null;
		
		for(int k=1; k< nodes.size(); k++)
		{
			f = functions.get(k-1);
			i = new Interval(nodes.get(k-1), nodes.get(k));
			if(i.belongsToMe(x))
			{
				return f;
			}
		}
		
		return null;
	}
	
	public Interval getIntervalForX(double x)
	{
		Interval i = null;
		for(int k=1; k< nodes.size(); k++)
		{
			i = new Interval(nodes.get(k-1), nodes.get(k));
			if(i.belongsToMe(x))
			{
				return i;
			}
		}
		
		return null;
	}
	public int isInternalNode(double x)
	{
		if(isExtreme(x))
			return -1;
		else
		 return isNode(x);
	}
	
	public int isNode(double x)
	{
		boolean isNode = false;
		int n = 0;
		for(Double d: nodes)
		{
			isNode = x == d;
			if(isNode)
				return n;
			n++;
		}
		return -1;
	}
	
	public boolean isExtreme(double x)
	{
		boolean isExtreme = false;
		isExtreme = x == nodes.get(0) || x == nodes.get(nodes.size() -1);
		
		return isExtreme;
	}
	
	public int getFunctionsLength()
	{
		return this.functions.size();
	}

	public int getNodesLength()
	{
		return this.nodes.size();
	}


	public List<Double> getNodes() {
		return nodes;
	}


	public void setNodes(List<Double> nodes) {
		this.nodes = nodes;
	}


	public List<VariabledFunction> getFunctions() {
		return functions;
	}


	public void setFunctions(List<VariabledFunction> functions) {
		this.functions = functions;
	}
	
	public String getFunctionAt(int index)
	{
		String cad = "";
		for(VariabledFunction s: this.functions)
		{
			cad += " : " + s.getFunctionAt(0);
		}
		
		return cad;
	}	
	
	public boolean belongsToLp(Interval i, double p) throws Exception
	{
		
		int n=0;
		System.out.println("Viendo si estan en LP");
		
		
		for(VariabledFunction f : this.functions )
		{
			System.out.println("Viendo si estan en LP " +n);
			if(!f.belongsToLp(new Interval(this.nodes.get(n), this.nodes.get(n+1)), p))
				return false;
			n++;
		}
		return true;
	}
	public Interval getLastInterval()
	{
		return new Interval(nodes.get(nodes.size()-2), nodes.get(nodes.size()-1));
	}
	
	public String getFunctionStrAt(int index)
	{
		if(index < functions.size())
		{
			return functions.get(index).getFunctionAt(0);
		}
		
		return null;
	}	
	
	public String toString()
	{
		
		String s = "";
		for(int k=0; k< this.nodes.size()-1; k++)
		{
			s += "[a, b] -> [" + nodes.get(k) + ", " + nodes.get(k+1) + "]" +
			this.functions.get(k).getFunctionAt(0);
		}
		return s;
	}

}
