/*
    Copyright 2015 Carlos Toro Peas, Madrid, Espaa 
    This file is part of Android Integral Calculator. 
    Inherted from MaximaOnAndroid project whose
    Copyright 2012, Yasuaki Honda (yasuaki.honda@gmail.com)
    
    Android Integral Calculator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Android Integral Calculator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.carlos.math.functions;

import java.util.ArrayList;
import java.util.List;

import com.numerico.algebralineal.Util;


public class ExtendedVariabledFunction extends VariabledFunction
{

	private List<Double> nodes;
	private List<VariabledFunction> functions;
	
	public ExtendedVariabledFunction(String []sortedFunctions, double []sortedNodes) 
	{
		super();
		this.dimension = 1;
		this.functions = new ArrayList<VariabledFunction>();
		nodes = new ArrayList<Double>();
		for(String f : sortedFunctions)
		{
			this.addFunction(f);
		}
		for(Double n : sortedNodes)
		{
			this.nodes.add(n);
		}
	}
	
	
	public ExtendedVariabledFunction differenciate(String variable)
	{
		ExtendedVariabledFunction ef ;
		List<VariabledFunction> derivates = new ArrayList<VariabledFunction>();
		
		for(VariabledFunction f : this.functions)
		{
			derivates.add(f.differenciate(variable));
		}
		
		return new ExtendedVariabledFunction(derivates, this.nodes);
	}
	
	public ExtendedVariabledFunction(List<VariabledFunction> sortedFunctions, List<Double> sortedNodes) 
	{
		super();
		this.dimension = 1;
		this.functions = sortedFunctions;
		nodes = sortedNodes;
	}
	
	public double integrate() throws Exception
	{
		double limit_inf, limit_sup, result = 0;
		limit_inf = this.nodes.get(0);
		limit_sup = this.nodes.get(1);
		int k = 0;
		for(VariabledFunction f: this.functions)
		{
			limit_sup = this.nodes.get(k+1);
			result += f.integrates(limit_inf, limit_sup);
			limit_inf = limit_sup;
			k++;
		}
		
		return result;
	}
	
	public void addFunction (String function)
	{
		this.functions.add(new VariabledFunction(function));
	}
	
	
	public double [] f(double x, double[] y)
	{
		Interval i = null;
		VariabledFunction f = null;
		
		for(int k=1; k< nodes.size(); k++)
		{
			f = functions.get(k-1);
			
			i = new Interval(nodes.get(k-1), nodes.get(k));
			if(i.belongsToMe(x))
			{
				return f.f(x, y);
			}
			
		}
		
		return y;
	}

	@Override	
	public boolean isExtendedFunction()
	{
		return true;
	}
	
	public int getFunctionIndexForX(double x)
	{
		Interval i = null;
		VariabledFunction f = null;
		
		for(int k=1; k< nodes.size(); k++)
		{
			f = functions.get(k-1);
			i = new Interval(nodes.get(k-1), nodes.get(k));
			if(i.belongsToMe(x))
			{
				return k-1;
			}
		}
		return 0;
	}
	public VariabledFunction getFunctionForX(double x)
	{
		Interval i = null;
		VariabledFunction f = null;
		
		for(int k=1; k< nodes.size(); k++)
		{
			f = functions.get(k-1);
			i = new Interval(nodes.get(k-1), nodes.get(k));
			if(i.belongsToMe(x))
			{
				return f;
			}
		}
		
		return null;
	}
	

	public int isInternalNode(double x)
	{
		if(isExtreme(x))
			return -1;
		else
		 return isNode(x);
	}
	
	public int isNode(double x)
	{
		boolean isNode = false;
		int n = 0;
		for(Double d: nodes)
		{
			isNode = x == d;
			if(isNode)
				return n;
			n++;
		}
		return -1;
	}
	
	public boolean isExtreme(double x)
	{
		boolean isExtreme = false;
		isExtreme = x == nodes.get(0) || x == nodes.get(nodes.size() -1);
		
		return isExtreme;
	}
	
	public int getFunctionsLength()
	{
		return this.functions.size();
	}

	public int getNodesLength()
	{
		return this.nodes.size();
	}


	public List<Double> getNodes() {
		return nodes;
	}


	public void setNodes(List<Double> nodes) {
		this.nodes = nodes;
	}


	public List<VariabledFunction> getFunctions() {
		return functions;
	}


	public void setFunctions(List<VariabledFunction> functions) {
		this.functions = functions;
	}
	
	public String getFunctionAt(int index)
	{
		String cad = "";
		for(VariabledFunction s: this.functions)
		{
			cad += " : " + s.getFunctionAt(0);
		}
		
		return cad;
	}	
	
	public String getFunctionStrAt(int index)
	{
		if(index < functions.size())
		{
			return functions.get(index).getFunctionAt(0);
		}
		
		return null;
	}	
	

}
