/*
    Copyright 2015 Carlos Toro Peas, Madrid, Espaa 
    This file is part of Android Integral Calculator. 
    Inherted from MaximaOnAndroid project whose
    Copyright 2012, Yasuaki Honda (yasuaki.honda@gmail.com)
    
    Android Integral Calculator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Android Integral Calculator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.carlos.math.numerico.fourier;

import java.util.ArrayList;
import java.util.List;

import com.carlos.math.functions.ExtendedFucntion;
import com.carlos.math.functions.Interval;
import com.carlos.math.functions.VariabledFunction;
import com.numerico.algebralineal.Util;

public class BaseSin 
{
	private int size;
	private List<String> sinSerie;
	private Interval interval;
	
	
	public BaseSin(int size, Interval i, int kind) 
	{
		this.size = size;
		sinSerie = new ArrayList<String>();
		this.interval = i;
		double divide = 2;
		if(kind != FourierSerie.KIND_FOURIER)
		{
			divide = 1;
		}
		for(int k=1; k<= size; k++)
		{
			sinSerie.add("sin("+ Util.PI + "*" + k + "x/" + (interval.getLength()/divide) + ")");
		}
	}
	

	
	public BaseSin(int size, int kind) 
	{
		this(size, new Interval(-Util.PI, Util.PI), kind);
	}

	public int getSize() {
		return size;
	}

	public void setSize(int size) {
		this.size = size;
	}

	public List<String> getSinSerie() {
		return sinSerie;
	}

	public void setSinSerie(List<String> sinSerie) {
		this.sinSerie = sinSerie;
	}

	public List<VariabledFunction> getListaXBaseOf(VariabledFunction f)
	{
		
		List<VariabledFunction> lista = new ArrayList<VariabledFunction>();
		if(!f.isExtendedFunction())
		{
			for(String ei: sinSerie)
			{
				lista.add(new VariabledFunction("(" + f.getFunctionAt(0) + ")*(" + ei + ")"));
			}
		}
		else
		{
			ExtendedFucntion ef = (ExtendedFucntion)f;
			List<VariabledFunction> listaFunctiones ;
			for(String ei: sinSerie)
			{
				listaFunctiones = new ArrayList<VariabledFunction>();
				//String [] efes = new String[ef.getFunctionsLength()];
				//int k = 0;
				for(VariabledFunction function : ef.getFunctions())
				{	
					listaFunctiones.add(new VariabledFunction("(" + function.getFunctionAt(0) + ")*(" + ei + ")"));
					
				}
				//System.out.println("");
				lista.add(new ExtendedFucntion(listaFunctiones, ef.getNodes()));
			}
		}
		return lista;
	}
	
	public static List<VariabledFunction> getBase(int size, int kind)
	{
		Interval i = new Interval(-Util.PI, Util.PI);
		
		return BaseSin.getBase(size, i, kind);
	}
	
	public static List<VariabledFunction> getBase(int size, Interval i, int kind)
	{
		List<VariabledFunction> lista = new ArrayList<VariabledFunction>();
		//lista.add(new Function(new String[]{"0.5"}, 1));
		double divide = 2;
		if(kind != FourierSerie.KIND_FOURIER)
		{
			divide = 1;
		}
		
		for(int k=1; k< size; k++)
		{
			lista.add(new VariabledFunction("sin("+ Util.PI + "*" + k + "x/" + (i.getLength()/divide) + ")"));
		}
		
		return lista;
	}

}
