/*
    Copyright 2015 Carlos Toro Peas, Madrid, Espaa 
    This file is part of Android Integral Calculator. 
    Inherted from MaximaOnAndroid project whose
    Copyright 2012, Yasuaki Honda (yasuaki.honda@gmail.com)
    
    Android Integral Calculator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Android Integral Calculator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.carlos.math.numerico.fourier;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;



import com.carlos.math.functions.ExtendedFucntion;
import com.carlos.math.functions.Interval;
import com.carlos.math.functions.LPException;
import com.carlos.math.functions.PointRepresentable;
import com.carlos.math.functions.VariabledFunction;
import com.numerico.algebralineal.Util;

public class FourierSerie
{
	public static final int KIND_SIN=1;
	public static final int KIND_COS=-1;
	public static final int KIND_FOURIER=0;
	private int kind;
	
	private List<VariabledFunction> fan;
	private List<VariabledFunction> fbn;
	
	private List<Coeficent> an;
	private List<Coeficent> bn;
	
	private VariabledFunction fa0;
	private Coeficent a0;
	
	private VariabledFunction function;
	private int size;
	private double sum;
	private boolean sumated;
	
	public static final double MIN_TOL = 1e-12;
	public static final double TOL_INI = 1e-6;
	private static final double ACCEPTABLE_DIFF = 1e-6;
	private Interval interval;
	double h0 = 0.01;
	
	public FourierSerie() 
	{
		an = new ArrayList<Coeficent>();
		bn = new ArrayList<Coeficent>();
		this.kind = 0;
		sum = 0;
		sumated = false;
		this.interval = new Interval(-Util.PI, Util.PI);
	}
	
	public FourierSerie(Interval i) 
	{
		this();
		this.interval = i;
	}
	
	public FourierSerie(VariabledFunction function) {
		this();
		this.function = function;
	}

	public FourierSerie(VariabledFunction function, int size, Interval interval, int kind) {
		this(function);
		this.size = size;
		this.interval = interval;
		switch (kind) {
		case 1:
			this.kind =1;
			break;
		case -1:
			this.kind =-1;
			break;
		default:
			this.kind =0;
			break;
		}
		this.kind = kind;
	}

	public Vector<Vector<Double>> getFormatedTable()
	{
		Vector<Vector<Double>> v = new Vector<Vector<Double>>();
		Vector<Double> w = new Vector<Double>();
		
		w.add((double)0);
		w.add(an.get(0).getValue());
		w.add(0.0);
		v.add(w);
		
		int k = 1;
		for(Coeficent c : this.bn)
		{
			w = new Vector<Double>();
			w.add((double)k);
			w.add(an.get(k).getValue());
			w.add(c.getValue());
			k++;
			v.add(w);
		}
		return v;
	}

	
	public void print()
	{
		System.out.println("Fourier serie of " + this.function);
		System.out.println(this.toString());
	}
	
	public String toFunction()
	{
		String s = "";
		for(VariabledFunction vf: getSeriesAsFunctions())
		{
			s += "+" + vf.getFunctionAt(0);
		}
		return s;
	}
	
	public VariabledFunction toVariabledFunctionFunction()
	{
		String s = "";
		for(VariabledFunction vf: getSeriesAsFunctions())
		{
			s += "+" + vf.getFunctionAt(0);
		}
		return new VariabledFunction(s);
	}
	
	public String toString()
	{
		String s ="";
		int k = 0;
		//s = a0.getName() + "= " + this.a0.getValue();
		for(VariabledFunction c : this.fan)
		{
			s +=  this.an.get(k).getValue() + "*" + c + "\r\n"; 
			k++;
		}
		k = 0;
		for(VariabledFunction c : this.fbn)
		{
			if(bn.size()>k)
			s += this.bn.get(k).getValue() + "*" +c + "\r\n";
			else
				s +=  "XXXXXX*" +c + "\r\n";
			k++;
		}
		
		return s;
	}
	public static void main(String args[])  throws Exception
	{
		/*
		VariabledFunction f2 = new VariabledFunction("arctan(x)");
		f2.addVariable("x");
		int size = 10;
		
		FourierSerie fu = new FourierSerie(f2, size, new Interval(-Util.PI, Util.PI));
		
		
		System.out.println("F " + fu.evaluateAt(0));
		System.exit(0);
		*/
		ExtendedFucntion f = new ExtendedFucntion(new String[]{"-x", "x"}, new double[]{-2, 0, 2});
		
		FourierSerie fu = f.getFourierSeries(10, new Interval(-2, 2), FourierSerie.KIND_SIN);
		
		fu.formateFunctions();
		
		fu.calculateCoeficents();
		System.out.println("****");
		for(VariabledFunction vf: fu.getSeriesAsFunctions())
		{
			System.out.println(vf.getFunctionAt(0));
		}
		System.out.println("****");
		System.out.println(fu.toVariabledFunctionFunction().f(2));
		/*
		System.out.println("10 " + f.sumaN(10));
		System.out.println("50 " + f.sumaN(50));
		System.out.println("F " + f.evaluateAt(2));
		f.print();
		
		/*
		System.out.println("60 " + f.sumaN(60));
		System.out.println("70 " + f.sumaN(70));
		System.out.println("8100 " + f.sumaN(100));
		System.out.println("150 " + f.sumaN(150));
		System.out.println("200 " + f.sumaN(200));
		System.out.println("300 " + f.sumaN(300));
		System.out.println("400 " + f.sumaN(400));
		System.out.println("500 " + f.sumaN(500));
		System.out.println("600 " + f.sumaN(600));
		System.out.println("700 " + f.sumaN(700));
		System.out.println("800 " + f.sumaN(800));
		System.out.println("900 " + f.sumaN(900));
		System.out.println("1000 " + f.sumaN(1000));
		System.out.println("2000 " + f.sumaN(2000));
		System.out.println("3000 " + f.sumaN(3000));
		System.out.println("10000 " + f.sumaN(1000));
		*/
	}

	
	public double sumaN(int n)
	{
		int m = Math.min(n, bn.size());
		
		if(!sumated)
		{
			
			sum = an.get(0).getValue();
			for(int k=0; k< m; k++)
			{
				sum += (an.get(k).getValue() + bn.get(k).getValue());
			}
			sumated = true;
		}
		return sum;
	}
	
	public void formateFunctions()
	{
		/*Base del seno*/
		fbn = new BaseSin(size, interval, this.kind).getListaXBaseOf(function);
		/*Base del Coseo*/
		fan = new BaseCos(size, interval, this.kind).getListaXBaseOf(function);
		/*IndepTerm*/
		
		if(this.function.isExtendedFunction())
		{
			fa0 = new VariabledFunction(new String[]{"0"});
		}
		else
		{
			fa0 = new VariabledFunction(new String[]{this.function.getFunctionAt(0)});
		}
	}
	public void calculateCoeficents() throws LPException
	{
		an.clear();
		bn.clear();
		
		double add = 0;
		int contador = 1;
		double multiplies = 1.0;
		Interval i2 = interval;
		if(kind != FourierSerie.KIND_FOURIER)
		{
			multiplies = 2.0;
			//i2 = new Interval((interval.getSup()+interval.getInf())/2, interval.getSup());
		}
		
		//multiplies = 1.0;
		for(VariabledFunction f: fbn)
		{
			if(kind != FourierSerie.KIND_COS)
			{
				try {
					add = f.integrates(i2);
				} catch (LPException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					throw e;
				}
				add = FourierSerie.numeralize(add / (interval.getLength()/2.0));
			}
			this.bn.add(new Coeficent("b" + contador, add));
			contador ++;
		}
		
		
		add = 0;
		contador = 0; 
		
		for(VariabledFunction f: fan)
		{
			if(kind != FourierSerie.KIND_SIN)
			{
				try {
					add = f.integrates(i2);
				} catch (LPException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					throw e;
				}
				add = FourierSerie.numeralize(add / (interval.getLength()/2.0));
			}
			this.an.add(new Coeficent("a" + contador, add));
			contador ++;
		}
	}

	public List<VariabledFunction> getFan() {
		return fan;
	}

	public void setFan(List<VariabledFunction> fan) {
		this.fan = fan;
	}

	public List<VariabledFunction> getFbn() {
		return fbn;
	}

	public void setFbn(List<VariabledFunction> fbn) {
		this.fbn = fbn;
	}

	public VariabledFunction getFunction() {
		return function;
	}

	public void setFunction(VariabledFunction function) {
		this.function = function;
	}

	public int getSize() {
		return size;
	}

	public void setSize(int size) {
		this.size = size;
	}

	public List<Coeficent> getAn() {
		return an;
	}

	public List<Coeficent> getBn() {
		return bn;
	}

	public double getSum() {
		return sum;
	}

	public boolean isSumated() {
		return sumated;
	}
	
	public static double numeralize(double d)
	{
		int [] integers = {0, 1, -1, 2, -2 , 3, -3, 4, -4,  5, -5, 6, -6, 7, -7, 8, -8, 9, -9};
		for(int i: integers)
		{
			if(Math.abs(d-i) < FourierSerie.ACCEPTABLE_DIFF)
			{
				return i;
			}
		}
		
		return d;
	}
	
	public List<VariabledFunction> getCosSeries()
	{
		List<VariabledFunction> cosSerieReturn = new ArrayList<VariabledFunction>();
		List<VariabledFunction> cosSerie = BaseCos.getBase(bn.size(), this.interval, this.kind);
		int k=0;
		for (VariabledFunction f: cosSerie)
		{
			cosSerieReturn.add(new VariabledFunction(an.get(k).getValue() + "*(" + f.getFunctionAt(0) + ")"));
			k++;
		}
		
		return cosSerieReturn;
	}
	
	public List<VariabledFunction> getSinSeries()
	{
		List<VariabledFunction> sinSerieReturn = new ArrayList<VariabledFunction>();
		List<VariabledFunction> sinSerie = BaseSin.getBase(an.size(), this.interval, this.kind);
		int k=0;
		
		for (VariabledFunction f: sinSerie)
		{
			if(k<bn.size())
				sinSerieReturn.add(new VariabledFunction(bn.get(k).getValue() + "*(" + f.getFunctionAt(0) + ")"));
			k++;
		}
		
		return sinSerieReturn;
	}
	
	List<VariabledFunction> fourierSerieFunction;
	
	
	public List<VariabledFunction> getSeriesAsFunctions()
	{
		if(fourierSerieFunction != null)
		{
			return fourierSerieFunction;
		}
		
		fourierSerieFunction = new ArrayList<VariabledFunction>();
		List<VariabledFunction> sinSerieReturn = this.getSinSeries();
		List<VariabledFunction> cosSerieReturn = this.getCosSeries();
		
		for(int k = 0; k< cosSerieReturn.size(); k++)
		{
			fourierSerieFunction.add(cosSerieReturn.get(k));
			//System.out.println("adding cos function " + cosSerieReturn.get(k).getFunctionAt(0));
			
			if((k<sinSerieReturn.size()))
			{
				fourierSerieFunction.add(sinSerieReturn.get(k));
		//		System.out.println("adding sin function " + sinSerieReturn.get(k).getFunctionAt(0));
				
			}
		}
		
		return fourierSerieFunction;
	}
	
	public double evaluateAt(double x)
	{
		
		List<VariabledFunction> fourierSerie = this.getSeriesAsFunctions();
		double returnD = 0;
		//String ss="";
		int k=0;
		for(VariabledFunction f: fourierSerie)
		{
			
			returnD += f.f(x, new double[1])[0];
			k++;
		}
		return returnD;
	}
	
	public List<PointRepresentable> getRepresentablePoints(Interval interval, double size) throws Exception
	{
		List<PointRepresentable> pointList = new ArrayList<PointRepresentable>();
		PointRepresentable p = null;
		double x = interval.getInf();
		do
		{
			p = new PointRepresentable(x, this.evaluateAt(x));
			pointList.add(p);
			x += size; 
			
		}while(x <= interval.getSup());
		
		return pointList;
	}
	
	public Coeficent getNthCoefficientAnAt(int index)
	{
		return an.get(index);
	}
	public Coeficent getNthCoefficientBnAt(int index)
	{
		return bn.get(index);
	}

	public Interval getInterval() {
		return interval;
	}

	public void setInterval(Interval interval) {
		this.interval = interval;
	}
	
	
}
