/*
    Copyright 2015 Carlos Toro Peas, Madrid, Espaa 
    This file is part of Android Integral Calculator. 
    Inherted from MaximaOnAndroid project whose
    Copyright 2012, Yasuaki Honda (yasuaki.honda@gmail.com)
    
    Android Integral Calculator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Android Integral Calculator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.carlos.math.numerico.rkuta;

import com.carlos.math.functions.VariabledFunction;


public class Solucion 
{
	private double t;
	private double [] y;	
	private double h;
	private double [] error;
	private VariabledFunction solucion;
	private RungeKutta r;
	private int nEval;
	private int rejected;
	public Solucion() {
		
	
	}
	public Solucion(double t, double y [], double h, VariabledFunction f, RungeKutta r) 
	{
		this.solucion = f;
		this.y = new double[y.length];
		for(int k=0; k< y.length; k++)
		this.y[k] = y[k];
		this.t = t;
		this.h = h;
		this.calculaError();
		this.r = r;
		this.nEval = r.getNeval();
		this.rejected = r.getRejected();
	}
	
	public String toString()
	{
		String cadena = "Evaluado en " + t;
		for(int k=0; k< y.length; k++)
		{
			if(error != null )
				cadena += "\n" +  (k + ": Y " + y[k] + "\tError: " + error[k]);
			else
				cadena += "\n" + (k + ":  " + y[k]);
		}
		return cadena;
	}
	
	public void write() 
	{
		//System.out.println(this.toString());
	}
	
	public double getMaxError()
	{
		double d = 0;
		for(double max: error)
		{
			d = Math.max(d, max); 
		}
		
		return Math.abs(d);
	}
	
	public void  calculaError()
	{
		//System.out.println(SimplexUtil.printVector2("XXXX", y));
		//System.out.println(SimplexUtil.printVector2("YYY", this.solucion.f(t, y)));
		error = new double [y.length];
		if(solucion == null)
		{
			for(int k=0; k< y.length; k++)
			{
				error [k] = 0;
			}
			return;
		}

		double [] z = new double [y.length];

		if(solucion.isNullFunction())
		{
			for(int k=0; k< y.length; k++)
			{
				z[k] = 0;
			}
		}
		else
		{
			z = solucion.f(t, new double[y.length]);
			/*for(int k=0; k< y.length; k++)
			{
				try
				{
					
					z[k] = solucion == null || solucion.getFunctionAt(k) == null || solucion.getFunctionAt(k).equals("null") ? 0 : this.solucion.f(t, new double[1])[k];
					System.out.println("Para calcular ["+k+"]  "+ solucion.getFunctionAt(k) + " " + z[k] + " - " + y[k] );

				}
				catch (Exception e) {
					z[k] = 0;
					e.printStackTrace();
				}
			}
			*/
		}
		for(int k=0; k< y.length; k++)
		{
			error[k] = Math.abs(z[k] - y[k]);
			//System.out.println("Erro k" + error[k]);

		}
	}
	public double getT() {
		return t;
	}
	public void setT(double t) {
		this.t = t;
	}
	public VariabledFunction getSolucion() {
		return solucion;
	}
	public void setSolucion(VariabledFunction solucion) {
		this.solucion = solucion;
	}
	public double[] getY() {
		return y;
	}
	public double[] getError() {
		return error;
	}
	public double getH() {
		return h;
	}
	public void setH(double h) {
		this.h = h;
	}
	public void setY(double[] y) {
		this.y = y;
	}
	public RungeKutta getR() {
		return r;
	}
	public void setR(RungeKutta r) {
		this.r = r;
	}
	public int getnEval() {
		return nEval;
	}
	public int getRejected() {
		return rejected;
	}
	
	
}
