/*
    Copyright 2015 Carlos Toro Peñas, Madrid, España 
    This file is part of Android Integral Calculator. 
    Inherted from MaximaOnAndroid project whose
    Copyright 2012, Yasuaki Honda (yasuaki.honda@gmail.com)
    
    Android Integral Calculator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Android Integral Calculator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.mathstools.keyboard;


import com.mathstools.integral.MainActivity;
import com.mathstools.integral.utils.Utils;
import com.mathstools.integral.R;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.inputmethodservice.InputMethodService;
import android.inputmethodservice.KeyboardView;
import android.inputmethodservice.KeyboardView.OnKeyboardActionListener;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnFocusChangeListener;
import android.view.View.OnTouchListener;
import android.view.ViewTreeObserver.OnWindowFocusChangeListener;
import android.view.inputmethod.InputMethodManager;

import android.webkit.WebView;
import android.widget.AutoCompleteTextView;
import android.widget.EditText;

@SuppressLint("NewApi") public class KeyboardListener extends InputMethodService implements OnKeyboardActionListener, OnFocusChangeListener, OnClickListener, OnTouchListener, OnWindowFocusChangeListener, Runnable
{

	private Object jsInterface = new Object()
	{
		@SuppressWarnings("unused")
		public void sendKeys() {
			getCurrentInputConnection().setComposingText("how do ya like me now?", 1);
		}
	};

	private MainActivity mainActivity = null;
	private View focusCurrent  = null;
	private KeyboardView mKeyboardView = null;
	private WebView webView = null;
	private EditText edittext = null;
	private String htmlId; 
	public KeyboardListener(View a, KeyboardView k, WebView webview, MainActivity mainActivity, String id)
	{
		this.mainActivity = mainActivity;
		this.mKeyboardView = k;
		this.webView = webview;
		htmlId = id;
	}

	public void hideCustomKeyboard() {
		//EditText editPrev= (EditText) mainActivity.findViewById(R.id.editPrew);
		//	editPrev.setVisibility(View.GONE);
		mKeyboardView.setVisibility(View.GONE);
		mKeyboardView.setEnabled(false);
		webView.loadUrl("javascript:hideError()");
	}

	@Override public void onPress(int arg0) {
		View focusCurrent = mainActivity.getCurrentFocus();
		if( focusCurrent==null || (focusCurrent.getClass()!=EditText.class && focusCurrent.getClass() != AutoCompleteTextView.class) ) return;
		edittext = (EditText) focusCurrent;

		int start = edittext.getSelectionStart();
		int end = edittext.getSelectionEnd();

		String text = edittext.getText().toString();
		text = text == null ? "" : text;

		if(start < end && arg0 != 39 && arg0 != 36)
		{
			text = text.substring(start,end);
			edittext.setText(text.substring(0, start)+text.substring(end, text.length()));
		}

		switch (arg0) {
		case 35:
			edittext.getText().insert(start, ".");
			break;
		case 31:
			edittext.getText().insert(start, "/");
			break;
		case 36:
			if(text.length()>0)
			{
				if(text.startsWith("-"))
				{
					edittext.getText().delete(0, 1);
				}
				else
					edittext.getText().insert(0, "-");
			}
			break;
		case 37:

			if(text.length()>0)
			{
				if(start == 0)
				{
					edittext.setText("");
					return;
				}
				edittext.getText().delete(start-1, start);
			}
			break;
		case 38:
			edittext.getText().clear();
			break;
		case 54: //inf
			edittext.getText().clear();
			edittext.getText().insert(0, "\u221e");
			break;
		case 39: // enter
			try 
			{
				edittext.setBackgroundColor(Color.parseColor("#b0bccd"));
				if(updateFunction())
				{
					hideCustomKeyboard();
				}
			} catch (Exception e) {
				// TODO Auto-generated catch block
				edittext.setBackgroundColor(Color.parseColor("#e01310"));
				this.mainActivity.showError(mainActivity.getKey("error_syntax"));
				edittext.requestFocus();
			}

			break;

		}
	}
	@Override
	public void onFocusChange(View v, boolean hasFocus) {
		// TODO Auto-generated method stub
		if( hasFocus ) 
		{
			showCustomKeyboard(v);
			this.mainActivity.setAutoList("%");
		}
		else
		{
			hideCustomKeyboard();
			if(!hasFocus || !this.edittext.hasFocus())
			{
				try 
				{
					boolean b = updateFunction();
					if(!b && edittext != null)
					{
						edittext.setBackgroundColor(Color.parseColor("#e01310"));
						this.mainActivity.showError(mainActivity.getKey("error_syntax"));
						//edittext.requestFocus();
					}
				} catch (Exception e) {

					// TODO Auto-generated catch block
					if(edittext != null)
					{
						edittext.setBackgroundColor(Color.parseColor("#e01310"));
						this.mainActivity.showError(mainActivity.getKey("error_syntax"));
						//edittext.requestFocus();
					}
				}	
			}
		}
	}

	private boolean updateFunction() throws Exception
	{


		switch (htmlId) {
		case "function":
			String fff =  Utils.rerereplace(edittext.getText().toString());
			if(Utils.validFunction(fff, mainActivity))
			{
				this.mainActivity.addAutoList(edittext.getText().toString());
				this.mainActivity.changeLabelFunction(fff.indexOf('y')>=0);
				edittext.setText(fff);
				webView.loadUrl("javascript:window.updateFunction('"+ fff +"', '" + htmlId + "')");
				return true;
			}
			break;

		case "ifunction1":
			fff =  Utils.rerereplace(edittext.getText().toString());
			if(Utils.validFunction(fff, mainActivity))
			{
				String s1= Utils.rerereplace(mainActivity.get_editTextEqn1().getText().toString());
				mainActivity.get_editTextEqn1().setText(s1);
				String s2= Utils.rerereplace(mainActivity.get_editTextEqn2().getText().toString());
				mainActivity.get_editTextEqn2().setText(s2);
				s1	=	s1 + " = " + s2;
				this.mainActivity.addAutoList(edittext.getText().toString());
				webView.loadUrl("javascript:window.updateFunction('"+ s1 +"', '" + htmlId + "')");
				return true;
			}
			break;
		case "ifunction2":
			fff =  Utils.rerereplace(edittext.getText().toString());
			if(Utils.validFunction(fff, mainActivity))
			{
				String s1= Utils.rerereplace(mainActivity.get_editTextEqn1().getText().toString());
				mainActivity.get_editTextEqn1().setText(s1);
				String s2= Utils.rerereplace(mainActivity.get_editTextEqn2().getText().toString());
				mainActivity.get_editTextEqn2().setText(s2);
				s1	=	s1 + " = " + s2;
				this.mainActivity.addAutoList(edittext.getText().toString());
				webView.loadUrl("javascript:window.updateFunction('"+ s1 +"', '" + htmlId + "')");
				return true;
			}
			break;
		default:
			fff =  edittext.getText().toString();
			if(fff.indexOf("-∞")>=0|| fff.indexOf("-inf")>=0 )
			{
				fff = "-inf";
			}
			else
			{
				if(fff.indexOf("∞")>=0 || fff.indexOf("inf")>=0)
				{
					fff = "inf";
				}
				else
				{
					double d = Utils.validValue(fff, mainActivity);
					fff = Double.toString(d);
				}
			}
			
			if(htmlId.equalsIgnoreCase("a"))
				mainActivity.get_editTexta().setText(fff);
			else
				mainActivity.get_ediTextb().setText(fff);
			webView.loadUrl("javascript:window.updateFunction('"+ fff +"', '" + htmlId + "')");
			return true;

			//break;
		}
		edittext.requestFocus();
		return false;
	}

	@Override public void onRelease(int primaryCode) {
		System.out.println("release " + primaryCode);
	}

	@Override public void onText(CharSequence text) {
		System.out.println("text " + text);
		focusCurrent = mainActivity.getCurrentFocus();
		if( focusCurrent==null || (focusCurrent.getClass()!=EditText.class && focusCurrent.getClass() != AutoCompleteTextView.class) ) return;

		EditText edittext = (EditText) focusCurrent;
		int start = edittext.getSelectionStart();
		edittext.getText().insert(start, text);
	}


	public void showCustomKeyboard( View v ) {
		//EditText editPrev= (EditText) mainActivity.findViewById(R.id.editPrew);
		//editPrev.setVisibility(View.VISIBLE);
		//		hideSoftKeyboard();
		mKeyboardView.setVisibility(View.VISIBLE);
		mKeyboardView.setEnabled(true);

		if( v!=null ) 
		{
			((InputMethodManager)mainActivity.getSystemService(Activity.INPUT_METHOD_SERVICE)).hideSoftInputFromWindow(v.getWindowToken(), 0);
		}
	}

	@Override public void swipeDown() {
	}

	@Override public void swipeLeft() {
	}

	@Override public void swipeRight() {
	}

	@Override public void swipeUp() {
	}

	@Override public void onKey(int primaryCode, int[] keyCodes) {

	}

	@Override
	public void onClick(View v) {
		// TODO Auto-generated method stub
		showCustomKeyboard(v);
		this.mainActivity.hideSoftKeyboard();
	}



	@Override
	public boolean onTouch(View v, MotionEvent event) {
		// TODO Auto-generated method stub
		this.mainActivity.hideSoftKeyboard();
		new Thread(this).start();
		return false;
	}

	@Override
	public void onWindowFocusChanged(boolean hasFocus) {
		// TODO Auto-generated method stub
		this.mainActivity.hideSoftKeyboard();

	}

	@Override
	public void run()
	{
		
		try {
			Thread.currentThread().sleep(100);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		this.mainActivity.hideSoftKeyboard();

	}

}
