/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2003-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.mathExpr.evaluator;

import de.jtem.mathExpr.Function;
//for doc-links:
import de.jtem.mathExpr.PredefinedFunction;
import de.jtem.mathExpr.parser.Expression;
import de.jtem.mathExpr.parser.FunctionCall;
import de.jtem.mathExpr.parser.List;


/**
 * This <code>abstract</code> class realises the inherited method {@link
 * #getEvaluator(FunctionCall,Context)} only, because a {@link FunctionCall}
 * evaluation is independent from the <code>Types</code> the parameters
 * are evaluated to.
 * <p>
 * If the <code>FunctionCall</code> "calls" a {@link PredefinedFunction},
 * the method {@link Type#getEvaluator(PredefinedFunction,Evaluator)}
 * will be called on the <code>Type</code> the parameter is evaluated to.
 * </p> 
 * 
 * @see FunctionCallEvaluator
 * @author marcel 
 */
public abstract class AbstractType implements Type
{
	/**
	 * Returns the <code>Evaluator</code> for the specified
	 * <code>FunctionCall</code> which parameter expressions
	 * can be several {@link Type Types} when evaluated. The appropriate
	 * {@link Function} to the <code>FunctionCall</code> will be get from the
	 * specified <code>Context</code>. The returned <code>Evaluator</code>
	 * will be a {@link FunctionCallEvaluator} in any case.
	 *  
	 * @see FunctionCall#getEvaluator(Type,Context)
	 * @param expr the <code>FunctionCall</code> the <code>Evaluator</code>
	 * is to be returned for.
	 * @param context the <code>Context</code>, the <code>Function<code> - the
	 * <code>FunctionCall</code> "is calling" - will be got from.
	 */
	public Evaluator getEvaluator(FunctionCall expr, Context context)
	{
		//the defModel contains the function in any case, because the parser
		//would not create a FunctionCall if the function does not exist
		Function f=context.getFunction(expr);
		int p=expr.getParameterCount();
		if(p==f.getParameterCount())
		{
			Expression[] parameter;
			if(p>1)
				parameter=((List)expr.getOperand()).getEntries();
			else
			{
				parameter=new Expression[1];
				parameter[0]=expr.getOperand();
			}
			return new FunctionCallEvaluator(f, parameter, this, context);
		}
		else
			throw new IllegalArgumentException(f+": "+
					f.getParameterCount()+" parameters expected, but "+
					+p+" parameters located.");
	}
}
