/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2003-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.mathExpr.evaluator.complexEvaluator;

import de.jtem.mathExpr.evaluator.Evaluator;
//for doc-links:
import de.jtem.mathExpr.parser.BinaryOperation;
import de.jtem.mfc.field.Complex;

/**
 * The {@link Evaluator} evaluating a {@link BinaryOperation} of type '-',
 * which both operands are {@link Complex} when evaluated.
 *
 * @see ComplexType#getEvaluator(BinaryOperation,Evaluator,Evaluator)
 * @see BinaryOperation#getType()
 */
public class ComplexMinusComplexEvaluator extends AbstractComplexEvaluator
{
	/**
	 * The <code>Evaluator</code> for the first operand of the {@link
	 * BinaryOperation}.
	 */
	protected Evaluator operand1Eval;
	/**
	 * The <code>Evaluator</code> for the second operand of the {@link
	 * BinaryOperation}.
	 */
	protected Evaluator operand2Eval;
	/**
	 * Creates a new <code>ComplexMinusComplexEvaluator</code> with the specified
	 * operand <code>Evaluators</code>. Those operand
	 * <code>Evaluators</code> provide the values to be subtracted.
	 * 
	 * @param e1 the <code>Evaluator</code> providing the minuend value.
	 * @param e2 the <code>Evaluator</code> providing the subtrahend value.
	 */
  public ComplexMinusComplexEvaluator(Evaluator e1, Evaluator e2)
  {
		operand1Eval=e1;
		operand2Eval=e2;
  }
	/**
	 * Executes the subtraction on the evaluation results of the given operand
	 * <code>Evaluators</code> and returns the result as a {@link Complex}
	 * <code>Object</code>.
	 * 
	 * @return the subtraction of both operands as a {@link Complex}
	 * <code>Object</code>.
	 * @see #getReturnType()
	 */
  public Object evaluate()
  {
    complexValue.assignMinus((Complex)operand1Eval.evaluate(),
        (Complex)operand2Eval.evaluate());
    return complexValue;
  }
}
