/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2003-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.mathExpr.evaluator.realVectorEvaluator;

import de.jtem.blas.RealVector;
//for tag-links:
import de.jtem.mathExpr.ListFormChecker;
import de.jtem.mathExpr.evaluator.Evaluator;
import de.jtem.mathExpr.evaluator.realEvaluator.Real;
import de.jtem.mathExpr.evaluator.realEvaluator.RealType;
import de.jtem.mathExpr.parser.List;

/**
 * The {@link Evaluator} evaluating a {@link List} representing a
 * {@link RealVector}. Any entry of that <code>List</code> must be
 * <code>Real</code> when evaluated.
 *
 * @see RealType#getEvaluator(List,Evaluator[])
 * @see ListFormChecker#representsVector(List)
 */
public class RealVectorEvaluator extends AbstractRealVectorEvaluator
{
	/**
	 * The <code>Evaluators</code> for the entries of the {@link
	 * List}.
	 */
	protected Evaluator[] entryEval;
	/**
	 * The <code>List</code> this <code>Evaluator</code> is
	 * evaluating.
	 */
	protected List list;
	/**
	 * Creates a new <code>RealVectorEvaluator</code> with the specified
	 * entry <code>Evaluators</code> and the <code>List</code> representing
	 * the {@link RealVector}. Those entry <code>Evaluators</code> provide the
	 * entry values for the vector.
	 * 
	 * @param expr the <code>List</code> to be evaluated.
	 * @param e the <code>Evaluator</code> array providing all vector entry
	 * values.
	 */
  public RealVectorEvaluator(List expr, Evaluator[] e)
  {
		list=expr;
		entryEval=e;
    realVector=new RealVector(expr.getEntryCount());
  }
	/**
	 * Fills a vector with the evaluation results of the given entry
	 * <code>Evaluators</code> and returns the result as a {@link RealVector}
	 * <code>Object</code>.
	 * 
	 * @return all entry values in a {@link RealVector}.
	 * @see #getReturnType()
	 */
  public Object evaluate()
  {
    for(int i=0; i<list.getEntryCount(); i++)
      realVector.set(i, ((Real)entryEval[i].evaluate()).getValue());
    return realVector;
  }
}
